unit main;
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Project:	TGIFImage demo application.                                   //
// Description:	Simple GIF Animation Builder.                                 //
// Copyright	(c) 1997-2008 Anders Melander.                                //
//		All rights reserved.                                          //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
interface

uses
  GIFImg,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls;

type
  TFormAnimate = class(TForm)
    ScrollBoxSource: TScrollBox;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    Image4: TImage;
    Image5: TImage;
    Image6: TImage;
    Panel1: TPanel;
    PanelPreview: TPanel;
    ImageAnimate: TImage;
    Panel3: TPanel;
    ButtonAnimate: TButton;
    ButtonSave: TButton;
    SaveDialog: TSaveDialog;
    GroupBox1: TGroupBox;
    CheckBoxCrop: TCheckBox;
    CheckBoxMerge: TCheckBox;
    CheckBoxPalette: TCheckBox;
    StatusBar1: TStatusBar;
    procedure ButtonAnimateClick(Sender: TObject);
    procedure ButtonSaveClick(Sender: TObject);
    procedure ImageClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormAnimate: TFormAnimate;

implementation

{$R *.DFM}


procedure TFormAnimate.FormCreate(Sender: TObject);
begin
  // Enable GIF animation and transparency
  GIFImageDefaultAnimate := True;
  GIFImageDefaultTransparent := True;

  // Avoid animation flickering by double buffering the TImage parent
  PanelPreview.DoubleBuffered := True;
end;

procedure TFormAnimate.ButtonAnimateClick(Sender: TObject);

  function TransparentIndex(Frame: TGIFFrame): byte;
  begin
    // Use the lower left pixel as the transparent color
    Result := Frame.Pixels[0, Frame.Image.Height-1];
  end;

  function AddBitmap(GIF: TGIFImage; Source: TGraphic; Transparent: boolean): TGIFFrame;
  var
    GCE: TGIFGraphicControlExtension;
    LoopExt: TGIFAppExtNSLoop;
  begin
    // Preview the image being added (for user feedback)
    ImageAnimate.Picture.Assign(Source);
    ImageAnimate.Update;

    // Add the source image to the animation
    Result := GIF.Add(Source);

    // Netscape Loop extension must be the first extension in the first frame!
    if (GIF.Images.Count = 1) then
    begin
      LoopExt := TGIFAppExtNSLoop.Create(Result);
      LoopExt.Loops := 0; // Number of loops (0 = forever)
    end;

    // Add Graphic Control Extension
    GCE := TGIFGraphicControlExtension.Create(Result);
    GCE.Delay := 30; // Animation delay (30 = 300 mS)
    if (Transparent) then
    begin
      GCE.Transparent := True;
      GCE.TransparentColorIndex := TransparentIndex(Result);
    end;
  end;

  function SizeOfGIF(GIF: TGIFImage): integer;
  var
    Stream: TStream;
  begin
    Stream := TMemoryStream.Create;
    try
      GIF.SaveToStream(Stream);
      Result := Stream.Size;
    finally
      Stream.Free;
    end;
  end;

var
  GIF: TGIFImage;
  i: integer;
  OptimizeOptions: TGIFOptimizeOptions;
  Size, SizeOptimized: integer;
begin
  Screen.Cursor := crHourGlass;
  try
    GIF := TGIFImage.Create;
    try
      // Add all the images in the scroll box to the animation
      for i := 0 to ScrollBoxSource.ControlCount-1 do
        // Add frame
        AddBitmap(GIF, // Destination GIF
          TImage(ScrollBoxSource.Controls[i]).Picture.Graphic, // Source bitmap
          (i > 0)); // First frame is not transparent

      // Calculate size of GIF before optimization
      Size := SizeOfGIF(GIF);

      // Optimize Color map...
      if (CheckBoxPalette.Checked) then
        GIF.OptimizeColorMap;

      // Optimize GIF frames...
      OptimizeOptions := [];
      if (CheckBoxMerge.Checked) then
        include(OptimizeOptions, ooMerge);
      if (CheckBoxCrop.Checked) then
        include(OptimizeOptions, ooCrop);
      if (OptimizeOptions <> []) then
        GIF.Optimize(OptimizeOptions, rmNone, dmNearest, 0);

      // Calculate size of GIF after optimization
      SizeOptimized := SizeOfGIF(GIF);

      // Display statistics
      StatusBar1.Panels[0].Text := Format('Size: %.0n bytes', [Size*1.0]);
      StatusBar1.Panels[1].Text := Format('Optimized: %.0n bytes', [SizeOptimized*1.0]);
      StatusBar1.Panels[2].Text := Format('Saved: %.1n%%', [(Size-SizeOptimized)*100/Size]);

      // Display the animation
      ImageAnimate.Picture.Assign(GIF);
    finally
      GIF.Free;
    end;
    ButtonSave.Enabled := True;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TFormAnimate.ButtonSaveClick(Sender: TObject);
begin
  // Set file extension and filter to GIF (or whatever is currently displayed)
  SaveDialog.DefaultExt := GraphicExtension(TGraphicClass(ImageAnimate.Picture.Graphic.ClassType));
  SaveDialog.Filter :=  GraphicFilter(TGraphicClass(ImageAnimate.Picture.Graphic.ClassType));
  // Prompt for filename
  if (SaveDialog.Execute) then
    ImageAnimate.Picture.SaveToFile(SaveDialog.Filename);
end;

procedure TFormAnimate.ImageClick(Sender: TObject);
begin
  // Preview a source image
  ImageAnimate.Picture.Assign(TImage(Sender).Picture);
end;

end.

